%This function exports all movement data to a large Excel file.  The user
%can choose which worms to analyze, the frame ranges, which analyses to
%perform, and the relevant options.
%
%batchAnalyze(params,options,analyses,savefile)
%
%Input must be in a strictly standardized format, consisting of three
%structures (params, options, analyses) and a string (savefile).
%
%'params' is a structure array containing three fields: splineFile, stageFile,
%and frames.  splineFile and stageFile are simply the full filepath strings
%to the respective file locations.  frames is numeric--if it equals 0, then
%all frames are analyzed.  If it is a two-element array, the first element
%is the starting frame to be analyzed, and the second element is the ending
%point.  It can be created with the 'struct' command.  You can include as
%many recordings as you want.
%
%'options' is a structure containing the fields, 'frameRate', 
%'bendNumbers', 'pointNumbers', 'calibration', and 'trackingStyle'.  These
%should all be numeric.  Calibration should be entered as a two-element
%array, with the first element as the x-value and the second element as the
%y-value.  Tracking Style refers to whether the new or old file format will
%be used--enter 1 for the pre 6-24-10 format, or 1 for the newer format.
%bendNumbers and pointNumbers should simply be arrays, such as [1 2 3 4 5].
% Note that the value 0 denotes the centroid for the pointNumbers variable.
%
%'analyses' is a structure containing the fields 'sob' (avg. sum of bends),
%'freq' (bending frequency), 'rms' (root mean square of bends), 'mb'
%(maximum sweep of bend), 'amp' (avg. amplitude of bends), 'spd' (average
%speed of point), 'dist' (average distance travelled by a point), and 'dir'
%(direction metrics).  Set the field to equal 0 if you don't want this
%analysis to be performed, and 1 if you do.
%
%'savefile' is just a string pointing to the tab-delimited .txt file you 
%want to save to.

function batchAnalyze(params,options,analyses,savefile)

frameRate=options.frameRate;
bendNumbers=options.bendNumbers;
pointNumbers=options.pointNumbers;  %0 is cog
calibration=options.calibration; calibx=calibration(1); caliby=calibration(2);
trackingStyle=options.trackingStyle;    %0 is new; 1 is old;

% results=zeros(length(params),16);

hugeDataMatrix=[];

for i=1:length(params) %for each worm recording
    splineFile=params(i).splineFile;
    stageFile=params(i).stageFile;
    framesToAnalyze=params(i).frames;
    
    if framesToAnalyze==0
        framesToAnalyze='all';
        cmt={'all'};
    else
        cmt={['Frm ' num2str(framesToAnalyze(1)) '-' num2str(framesToAnalyze(2))]};
    end
    lastSlash=find(splineFile=='\');
    lastSlash=lastSlash(end);
    name={splineFile(lastSlash+1:end-4)};
    
    spline=load(splineFile);
    if strcmp(framesToAnalyze,'all')
        numberOfRecordings=length(spline(:,1))/2;
    else
        numberOfRecordings=framesToAnalyze(2)-framesToAnalyze(1)+1;
    end
    lengthOfRecording=numberOfRecordings/frameRate;
    lengthOfRecording={num2str(lengthOfRecording)};
    
    if analyses.sob
        sob=averageSumOfBends(splineFile,framesToAnalyze);
        sob={num2str(sob)};
    else
        sob={'n/a'};
    end
    
    if analyses.dir
        [dir,dt]=wormLocomotion('Direction',splineFile,stageFile,frameRate,calibx,caliby,0,trackingStyle,framesToAnalyze);
        sf=dir(1);
        sf={num2str(sf)};
        sb=dir(2);
        sb={num2str(sb)};
        df=dt(1);
        df={num2str(df)};
        db=dt(2);
        db={num2str(db)};
        totDist=sum(dt);
        pf=dt(1)/totDist;
        pf={num2str(pf)};
        pb=dt(2)/totDist;
        pb={num2str(pb)};
    else
        sf={'n/a'};
        sb={'n/a'};
        df={'n/a'};
        db={'n/a'};
        pf={'n/a'};
        pb={'n/a'};
    end
    
    freq=[];
    rms=[];
    mb=[];
    for j=1:length(bendNumbers)
        if analyses.freq
            [freqTemp,time,head]=fftscript(splineFile,bendNumbers(j),frameRate,framesToAnalyze);
            freqTemp={num2str(freqTemp)};
        else
            freqTemp={'n/a'};
        end
        freq=[freq freqTemp];

        if analyses.rms
            rmsTemp=ang(splineFile,bendNumbers(j),framesToAnalyze);
            rmsTemp={num2str(rmsTemp)};
        else
            rmsTemp={'n/a'};
        end
        rms=[rms rmsTemp];
        
        if analyses.mb
            [f,time,head]=fftscript(splineFile,bendNumbers(j),frameRate,framesToAnalyze);
            mbTemp=bendCursor_gui(time,head);
            mbTemp={num2str(mbTemp)};
        else
            mbTemp={'n/a'};
        end
        mb=[mb mbTemp];
    end
    
    amp=[];
    alratio=[];
    spd=[];
    dist=[];
%     sf=[];
%     sb=[];
%     df=[];
%     db=[];
%     pf=[];
%     pb=[];

        if analyses.amp
            [ampTemp,alratioTemp]=wormLocomotion('Amplitude',splineFile,stageFile,frameRate,calibx,caliby,0,trackingStyle,framesToAnalyze);
            ampTemp=mean(ampTemp);
            ampTemp={num2str(ampTemp)};
            alratioTemp={num2str(alratioTemp)};
        else
            ampTemp={'n/a'};
            alratioTemp={'n/a'};
        end
        
        if analyses.dist
            [distTemp, ndTemp]=wormLocomotion('Distance',splineFile,stageFile,frameRate,calibx,caliby,0,trackingStyle,framesToAnalyze);
            ndTemp={num2str(ndTemp)};
        else
            ndTemp={'n/a'};
        end

        amp=ampTemp;
        alratio=alratioTemp;
        nd=ndTemp;
%         amp=[amp ampTemp];
%         alratio=[alratio alratioTemp];
        
    for k=1:length(pointNumbers)
%         if pointNumbers(k)==0
%             pn='cog';
%         else
%             pn=pointNumbers(k);
%         end
        pn=pointNumbers(k);
        

        
        if analyses.spd
            spdTemp=wormLocomotion('Speed',splineFile,stageFile,frameRate,calibx,caliby,pn,trackingStyle,framesToAnalyze);
            spdTemp=mean(spdTemp);
            spdTemp={num2str(spdTemp)};
        else
            spdTemp={'n/a'};
        end
        spd=[spd spdTemp];
        
        if analyses.dist
            [distTemp, dtTemp]=wormLocomotion('Distance',splineFile,stageFile,frameRate,calibx,caliby,pn,trackingStyle,framesToAnalyze);
            distTemp=sum(distTemp);
            distTemp={num2str(distTemp)};
        else
            distTemp={'n/a'};
        end
        dist=[dist distTemp];
        
%         if analyses.dir
%             [dir,dt]=wormLocomotion('Direction',splineFile,stageFile,frameRate,calibx,caliby,pn,trackingStyle,framesToAnalyze);
%             sfTemp=dir(1);
%             sfTemp={num2str(sfTemp)};
%             sbTemp=dir(2);
%             sbTemp={num2str(sbTemp)};
%             dfTemp=dt(1);
%             dfTemp={num2str(dfTemp)};
%             dbTemp=dt(2);
%             dbTemp={num2str(dbTemp)};
%             totDist=sum(dt);
%             pfTemp=dt(1)/totDist;
%             pfTemp={num2str(pfTemp)};
%             pbTemp=dt(2)/totDist;
%             pbTemp={num2str(pbTemp)};
%         else
%             sfTemp={'n/a'};
%             sbTemp={'n/a'};
%             dfTemp={'n/a'};
%             dbTemp={'n/a'};
%             pfTemp={'n/a'};
%             pbTemp={'n/a'};
%         end
%         sf=[sf sfTemp];
%         sb=[sb sbTemp];
%         df=[df dfTemp];
%         db=[db dbTemp];
%         pf=[pf pfTemp];
%         pb=[pb pbTemp];
    end
    
    hugeDataLine=[name cmt lengthOfRecording sob freq rms mb amp alratio spd nd dist sf sb df db pf pb];
    hugeDataMatrix=[hugeDataMatrix;hugeDataLine];
end

%put together all of the labels
nameLabel={'Name'};
commentsLabel={'Comments'};
lengthLabel={'Rec. Length'};
sobLabel={'Avg. Sum of Bends'};
freqLabel=[];
rmsLabel=[];
mbLabel=[];
for i=1:length(bendNumbers)
    freqLabel=[freqLabel {['Freq B' num2str(bendNumbers(i))]}];
    rmsLabel=[rmsLabel {['RMS B' num2str(bendNumbers(i))]}];
    mbLabel=[mbLabel {['Max Bend B' num2str(bendNumbers(i))]}];
end
% ampLabel=[];
% alratioLabel=[];
ampLabel={'Amp'};
alratioLabel={'A/L'};
spdLabel=[];
distLabel=[];
ndLabel={'Net distance traveled'};
sfLabel={'Steps F'};
sbLabel={'Steps B'};
dfLabel={'Dist F'};
dbLabel={'Dist B'};
pfLabel={'%% F'};
pbLabel={'%% B'};

for j=1:length(pointNumbers)
%     ampLabel=[ampLabel {['Amp P' num2str(pointNumbers(j))]}];
%     alratioLabel=[alratioLabel {['A/L P' num2str(pointNumbers(j))]}];
    spdLabel=[spdLabel {['Speed P' num2str(pointNumbers(j))]}];
    distLabel=[distLabel {['Dist P' num2str(pointNumbers(j))]}];
%     sfLabel=[sfLabel {['Steps F P' num2str(pointNumbers(j))]}];
%     sbLabel=[sbLabel {['Steps B P' num2str(pointNumbers(j))]}];
%     dfLabel=[dfLabel {['Dist F P' num2str(pointNumbers(j))]}];
%     dbLabel=[dbLabel {['Dist B P' num2str(pointNumbers(j))]}];
%     pfLabel=[pfLabel {['%% F P' num2str(pointNumbers(j))]}];
%     pbLabel=[pbLabel {['%% B P' num2str(pointNumbers(j))]}];
end

labels=[nameLabel commentsLabel lengthLabel sobLabel freqLabel rmsLabel mbLabel ampLabel alratioLabel...
    spdLabel ndLabel distLabel sfLabel sbLabel dfLabel dbLabel pfLabel pbLabel];

saveDataMatrix(labels,hugeDataMatrix,savefile)